C
C
C
      SUBROUTINE   INBCA
     I                  ( MESSFL, WDMFL, FE, DSN,
     I                    ABVAL, KNDX, INDEX, ABNDX, ABNAME,
     O                    RETC )
C
C     + + + PURPOSE + + +
C     Write basin characteristics attriubes to data set.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, WDMFL, FE, DSN, KNDX, INDEX(KNDX), ABNDX(200),
     $          RETC
      REAL      ABVAL(KNDX)
      CHARACTER*1 ABNAME(6,200)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     WDMFL  - Fortran unit number of the wdm data file
C     FE     - Fortran unit number for summary ouput,
C     DSN    - data-set number for the attributes
C     ABVAL  - values of attributes to be added
C     KNDX   - number of attributes to be added
C     INDEX  - array of pointers to the ABNDX array for attribute
C              index numbers
C     ABNDX  - array containing the corresponding attribute index
C              number for the 200 basin characteristics
C     ABNAME - array containing the corresponding attribute name
C              for the 200 basin characteristics
C     RETC   - return code
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   K, NDX, LEN, I, RET
C
C     + + + LOCAL DEFINITIONS + + +
C     RETC   - return code from putting attributes
C               -81 - data set does not exist
C              -102 - attribute already in data set
C              -103 - no room in data set for attribute
C              -104 - data present, can't update attribute
C              -109 - incorrect real value for attribute
C
C     + + + EXTERNALS + + +
      EXTERNAL  WDBSAR
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (//,2X, 'DSN', I6, ':    adding basin characteristics ',
     $               'attributes',
     $        //,2X, '              Name       Value' )
 2002 FORMAT (   2X, '              ', 6A1, 2X, G12.6 )
 2012 FORMAT (   2X, '    --->      ', 6A1, 2X, 'return code = ', I3,
     $               ',  value =', G12.6 )
C
C     + + + END SPECIFICATIONS + + +
C
C     write out heading and dsn
      WRITE (FE,2000) DSN
C
C     add basin characteristics attributes
      RETC = 0
      LEN = 1
      DO 100 K = 1, KNDX
        NDX = INDEX(K)
        CALL WDBSAR (WDMFL, DSN, MESSFL, ABNDX(NDX), LEN, ABVAL(K),
     O               RET )
        IF (RET .EQ. 0) THEN
C         document the add
          WRITE (FE,2002) (ABNAME(I,NDX),I=1,6), ABVAL(K)
        ELSE
C         error in putting attribute
          WRITE (FE,2012) (ABNAME(I,NDX),I=1,6), RET, ABVAL(K)
          RETC = RETC + 1
        END IF
 100  CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   INFREE
     I                    ( WDMFL, DSTYPE, DSN, DSNINC,
     O                      DSNF, RETC )
C
C     + + + PURPOSE + + +
C     Identify an unused data set in a wdm file.  Will start checking
C     at the data set number supplied in DSN.  The value of DSN will
C     be incremented by 1 untill an unused data set is found.  If no
C     free data set is found between DSN and 32000, the first free
C     data set between 1 and DSN will be returned.  If no free data
C     set can be found (data sets 1-3200 are already used!), a value
C     of zero will be returned for DSN.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMFL, DSTYPE, DSN, DSNINC, DSNF, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     WDMFL  - Fortran unit number of the wdm data file
C     DSTYPE - data set type
C              1 - time series      6 - rastor
C              2 - table            7 - space-time
C              3 - schematic        8 - attribute
C              4 - project          9 - message
C              5 - vector
C     DSN    - initial data set to try
C     DSNINC - increment to use for successive data set to try
C     DSNF   - free data set found, a value of zero indicates
C              no free data set was found
C     RETC   - return code
C              0 - found a free data set
C              -1 -unable to find a free data set
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   LOOP, MORE, RET, GPFLG, LREC, GRCNT, INC
C
C     + + + LOCAL DEFINITIONS + + +
C     LOOP   - infinite loop flag
C              0 - initial dsn supplied is between 1 and 32000, and
C                  routine is checking between dsn and 32000 for a
C                  free dsn
C              1 - either user supplied an invalid initial dsn or
C                  no free dsn was found between dsn and 32000, so
C                  program is now checking between 1 and dsn
C              2 - all data sets between 1 and 32000 have been checked,
C                  and no free dsn was found.
C     MORE   - flag to continue looking for free data set
C              0 - stop looking
C              1 - continue looking
C     GPFLG  - read(1)/write(2) flag
C     LREC   - record number of first record in data set (contains label)
C     GRCNT  - number of groups in data set
C     RET    - return code from call to wdscha
C                0 - data set exists and is correct DSTYPE
C              -81 - data set does not exist
C              -82 - data set exists, but is wrong DSTYPE
C              -84 - data set number out of range
C              -85 - trying to write to a read-only data set
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDSCHA
C
C     + + + END SPECIFICATIONS + + +
C
      LOOP = 0
      MORE = 1
      GPFLG = 2
C
C     start with supplied dsn and increment
      DSNF = DSN
      INC = DSNINC
      IF (INC .LT. 1  .OR.  INC .GT. 3000) THEN
C       outside valid range, use 1
        INC = 1
      END IF
      IF (DSNF .LE. 0  .OR.  DSNF .GT. 32000) THEN
C       outside valid range, start at 1
        DSNF = 1
        LOOP = 1
      END IF
 10   CONTINUE
C       check to see if the data set exists
        CALL WDSCHA ( WDMFL, DSNF, DSTYPE, GPFLG,
     O                LREC, GRCNT, RET )
        IF (RET .EQ. -81) THEN
C         data set does not exist, done
          MORE = 0
          RETC = 0
        ELSE IF (RET .EQ. 0  .OR.  RET .EQ. -85) THEN
C         data set exists or data set is read only
          DSNF = DSNF + INC
        ELSE IF (RET .EQ. -84) THEN
C         data set is out of valid range, start at 1
          DSNF = 1
          LOOP = LOOP + 1
          IF (LOOP .GT. 1) THEN
C           infinity check, have been past dsn 1 before, give it up
            MORE = 0
            RETC = -1
            DSNF = 0
            INC = 1
          END IF
        ELSE
C         unknown error, try next data set
          DSNF = DSNF + INC
        END IF
      IF (MORE .EQ. 1) GO TO 10
C
      RETURN
      END
C
C
C
      SUBROUTINE   INSVTS
     I                   ( WDMFL, DSN,
     O                     RETC )
C
C     + + + PURPOSE + + +
C     Saves the time-series array VAL to the WDM file.
C     Error messages are written to Fortran unit 99.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMFL, DSN, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     WDMFL  - Fortran unit number of the wdm file
C     DSN    - Data-set number for data
C     RETC   - Return code
C              0 - successful
C
C     + + + INCLUDES + + +
      INCLUDE 'ciowdm.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDTPUT, WDTPTQ
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (1X, 'Warning:  problem writing to wdm file    (insvts)',
     $      /,1X, '            dsn =', I5,
     $      /,1X, '          start =', I5, 5I3,
     $      /,1X, '           nval =', I5,
     $      /,1X, '           retc =', I5 )
C3000 format (' --> wdtptq:    dsn =', i5,
C    $      /,'             tsstep =', i5,
C    $      /,'             tunits =', i5,
C    $      /,'                ipt =', i5,
C    $      /,'             dtovwr =', i5,
C    $      /,'              dates =', i5, 5i3,
C    $      /,'                     ', i5, 5i3,
C    $      /,'                     ', i5, 5i3 )
C3001 format ('              val/q =', 4(F12.1,I3),
C    $     /,('                     ', 4(F12.1,I3) ) )
C
C     + + + END SPECIFICATIONS + + +
C
C     write data to wdm file
      IF (QFLAG .GE. 0) THEN
C       all data has same quality code
        CALL WDTPUT ( WDMFL, DSN, TSSTEP, DATES(1,1), IPT,
     I                DTOVWR, QFLAG, TUNITS, VAL,
     O                RETC )
      ELSE
C       quality code may vary
Ckmf
Ckmf    write(99,3000) dsn, tsstep, tunits, ipt, dtovwr, dates
Ckmf    write(99,3001) ( val(i), valq(i), i = 1, ipt )
        CALL WDTPTQ ( WDMFL, DSN, TSSTEP, DATES(1,1), IPT,
     I                DTOVWR, VALQ, TUNITS, VAL,
     O                RETC )
      END IF
      IF (RETC .NE. 0) THEN
C       error adding data to wdm file
        WRITE (99,2000) DSN, (DATES(I,1),I=1,6), IPT, RETC
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   WDTPTQ
     I                   ( WDMFL, DSN, DELT, DATE, NVAL, DTOVWR,
     I                     QUAL, TUNITS, VALU,
     O                     RETC )
C
C     + + + PURPOSE + + +
C     This routine writes a timeseries and its quality flags to a
C     WDM file.  Error messages are written to Fortran unit 99.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMFL, DSN, DELT, DATE(6), NVAL, DTOVWR, TUNITS, RETC
      INTEGER   QUAL(NVAL)
      REAL      VALU(NVAL)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     WDMFL  - Fortran unit number of WDM file
C     DSN    - WDM dataset number
C     DELT   - timestep in TUNITS units
C     DATE   - start date/time of array VALU
C     NVAL   - number of values to be written to WDM file
C     DTOVWR - data overwrite flag
C     QUAL   - array containing quality flags
C     TUNITS - time units code
C              1 - seconds       5 - months
C              2 - minutes       6 - years
C              3 - hours         7 - centuries
C              4 - days
C     VALU   - timeseries array to be written to WDM file
C     RETC   - return code from WDTPUT
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   LEN6, DATEW(6), DATEN(6), KNT, QFLAG, IPT, KPT, CONT
C
C     + + + EXTERNALS + + +
      EXTERNAL  COPYI, WDTPUT, TIMADD
C
C     + + + DATA INITIALIZATION + + +
      DATA  LEN6 / 6 /
C
C     + + + OUTPUT FORMATS + + +
 2001 FORMAT (10X,'in wdtptq, after call to wdtput',
     $      /,10X,'            wdmfl =', I5,
     $      /,10X,'              dsn =', I5,
     $      /,10X,'           tunits =', I5,
     $      /,10X,'             delt =', I5,
     $      /,10X,'           dtovwr =', I5,
     $      /,10X,'            qflag =', I5 )
 2002 FORMAT (10X,'             date =', I5, 5I3,
     $      /,10X,'              knt =', I5,
     $      /,10X,'              kpt =', I5,
     $      /,10X,'              ipt =', I5,
     $      /,10X,'             retc =', I5 )
C3002 FORMAT (10X,'           --> timadd: date, tunits, delt, knt')
C3003 FORMAT (10X,'           <-- timadd',
C    $      /,10X,'                       daten =', I5, 5I3,
C    $      /,10X,'                         kpt =', I5,
C    $      /,10X,'                       qflag =', I5 )
C
C     + + + END SPECIFICATIONS + + +
C
      RETC = 0
C     initialize date, flags, counters, and pointers
      CALL COPYI ( LEN6, DATE, DATEW )
      QFLAG = QUAL(1)
      KNT = 1
      KPT = 1
      IPT = 1
      CONT = 1
 100  CONTINUE
C       does time series need to be saved
        IF (IPT .EQ. NVAL) THEN
C         last time step, set write flag
          CONT = 3
        ELSE IF (QUAL(IPT+1) .NE. QFLAG) THEN
C         quality is changing set flag to save data with current flag
          CONT = 2
        END IF
        IF (CONT .GT. 1) THEN
C         save data for current quality flag
          CALL WDTPUT ( WDMFL, DSN, DELT, DATEW, KNT, DTOVWR,
     I                  QFLAG, TUNITS, VALU(KPT),
     O                  RETC )
          IF (RETC .NE. 0) THEN
C           error adding data to wdm file
            WRITE (99,2001) WDMFL, DSN, DELT, TUNITS, DTOVWR, QFLAG
            WRITE (99,2002) DATEW, KNT, KPT, IPT, RETC
Ckmf      else
Ckmf        no error
Ckmf        write (99,2001) wdmfl, dsn, delt, tunits, dtovwr, qflag
Ckmf        write (99,2002) datew, knt, kpt, ipt, retc
          END IF
          IF (CONT .EQ. 2) THEN
C           there is still more data, update date, flags, and pointers
Ckmf        write(99,3002) 
            CALL TIMADD ( DATEW, TUNITS, DELT, KNT, DATEN )
            CALL COPYI ( LEN6, DATEN, DATEW )
            KPT = IPT + 1
            QFLAG = QUAL(KPT)
            KNT = 0
            CONT = 1
Ckmf        write(99,3003) daten, kpt, qflag
          END IF
        END IF
C       update counter and array pointer
        KNT = KNT + 1
        IPT = IPT + 1
C       next time step if more data
      IF (CONT .EQ. 1) GO TO 100
C
      RETURN
      END
C
C
C
      SUBROUTINE   INSVTB
     I                   ( MESSFL, WDMFL, DSN,
     I                     MXCOL, MXROW, TRBUFF,
     O                     RETC )
C
C     + + + PURPOSE + + +
C     Saves the table array to a table data set.
C     Warning and error messages are written to Fortran unit 99.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, WDMFL, DSN, MXCOL, MXROW, RETC
      REAL      TRBUFF(MXCOL,MXROW)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of file containig the description
C              of the table template
C     WDMFL  - Fortran unit number of the wdm file
C     DSN    - data-set number for data
C     RETC   - return code
C              0 - successful
C
C     + + + INCLUDES + + +
      INCLUDE 'ciowtb.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      RET, TABID, ACLU, AGRP,
     $             TFLD, TNUM(4), TLEN(30), TCOL(30), TSPA,
     $             AFLD, ANUM(4), ALEN(30), ACOL(30), ASPA,
     $             FROW, NROWS, FFLD,
     $             TBCLU, TBGRP, DATFLG, TGRPPT
      CHARACTER*1  MFID(2), TTYP(30), ATYP(30)
      CHARACTER*16 TNAM
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDTBTM, WTBPUT
C
C     + + + OUTPUT FORMATS + + +
 2001 FORMAT (1X,'in insvtb, after call to wdtbtm',
     $      /,1X,'         ', I5, ' = dsn',
     $      /,1X,'         ', I5, ' = wdmfl',
     $      /,1X,'         ', I5, ' = tabid',
     $      /,1X,'         ', I5, ' = npt' )
 2002 FORMAT (1X,'         ', I5, ' = messfl',
     $      /,1X,'         ', 2A1, '= mfid'
     $      /,1X,'         ', I5, ' = tbclu',
     $      /,1X,'         ', I5, ' = tbgrp' )
 2003 FORMAT (1X,'         ', I5, ' = retc' )
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  MFID, TABID, TBCLU, TBGRP, DATFLG, FROW, FFLD
     $    / 'X','X',  1,    20,     1,      1,    1,    1 /
C
C     + + + END SPECIFICATIONS + + +
C
      RETC = 0
C     add table template to data set
      CALL WDTBTM ( MESSFL, MFID, TBCLU, TBGRP, WDMFL, DSN,
     I              TABID, NPT,
     O              TFLD, TNUM, TTYP, TLEN, TCOL, TSPA,
     O              TNAM, TGRPPT,
     O              AFLD, ANUM, ATYP, ALEN, ACOL, ASPA, ACLU, AGRP,
     O              RET )
      IF (RET .NE. 0) THEN
C       problem adding template
        RETC = RET
        WRITE (99,2001) DSN, WDMFL, TABID, NPT
        WRITE (99,2002) MESSFL, MFID, TBCLU, TBGRP
        WRITE (99,2003) RETC
      ELSE
C       template added, now add table
        NROWS = NPT
        TFLD = 14
        CALL WTBPUT ( WDMFL, DSN, TNAM, TABID, DATFLG,
     I                FROW, NROWS, FFLD, TFLD, TRBUFF,
     O                RET )
        IF (RET .NE. 0) THEN
C         problem adding table of data
          RETC = RET
        END IF
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   INITAT
     I                   ( VERS, SPACE, TIME )
C
C     + + + PURPOSE + + +
C     Initialize arrays for attribute identification.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   VERS, SPACE(5), TIME(7)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     VERS   - identifier for additional attributes to be added
C              0 - no additional attributes
C              1 - additional attributes generally associated with
C                  station header records
C              2 - additional attributes generally associated with
C                  basin characteristics
C              3 - additional attributes generally associtated with
C                  n-day records
C     SPACE  - defaults for space allocation
C              (1) - maximum number of attributes
C              (2) - maximum space for attributes
C              (3) - maximum number of data groups
C              (4) - number of up pointers
C              (5) - number of down pointers
C     TIME   - defaults for time series attributes
C              (1) - tcode      (5) - vbtime
C              (2) - tgroup     (6) - tsbyr
C              (3) - tsform     (7) - tsstep
C              (4) - compfg
C
C     + + + INCLUDES + + +
      INCLUDE 'catrib.inc'
C
C     + + + PARAMETERS + + +
      INTEGER   IMXS, RMXS, CMXS, IMXV, RMXV, CMXV, MXVS
      PARAMETER ( IMXS=13, RMXS=1, CMXS=3 )
      PARAMETER ( IMXV=6,  RMXV=7, CMXV=4 )
      PARAMETER ( MXVS=3 )
C
C     + + + DEFINITIONS + + +
C     IMXS   - number of shared integer attributes
C     RMXS   - number of shared real attributes
C     CMXS   - number of shared character attributes
C     IMXV   - maximum number of version specific integer attributes
C     RMXV   - maximum number of version specific real attributes
C     CMXV   - maximum number of version specific character attributes
C     MXVS   - number of recognized versions
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   INDXS(IMXS), RNDXS(RMXS), CNDXS(CMXS),
     $          CLENS(CMXS),
     $          INDX(IMXV,MXVS), RNDX(RMXV,MXVS), CNDX(CMXV,MXVS),
     $          CLEN(CMXV,MXVS), AMXV(3,MXVS),
     $          I, J, L, LPT, IMISS
      REAL      RMISS
      CHARACTER*1 CMISS
      CHARACTER*6 INAMS(IMXS), RNAMS(RMXS), CNAMS(CMXS),
     $            INAM(IMXV,MXVS), RNAM(RMXV,MXVS), CNAM(CMXV,MXVS)
C
C     + + + EXTERNALS + + +
      EXTERNAL   COPYI, ZIPI, ZIPR, ZIPC
C
C     + + + DATA INITIALIZATIONS + + +
C          integer attributes, shared
      DATA INAMS, INDXS
     $   / 'TCODE ', 'TGROUP', 'TSFORM', 'COMPFG', 'VBTIME', 'TSBYR ',
     $     'TSSTEP', 'space3', 'space4', 'space5', 'space1', 'space2',
     $     'ISTAID',
     $        17,       34,       84,       83,       85,       27,
     $        33,        0,        0,        0,        0,        0,
     $        51 /
C          real and character attributes, shared (name, index, length)
      DATA RNAMS, CNAMS, RNDXS, CNDXS, CLENS
     $   / 'TSFILL', 'STAID ', 'TSTYPE', 'STANAM',
     $        32,        2,        1,       45,
     $                  16,        4,       48 /
C           for each version, number of integer, real, & character
      DATA  AMXV / 6,7,4,   2,0,1,   4,4,0 /
C          additional integer attributes (name, index)
      DATA INAM, INDX
     $   / 'PARMCD', 'STATCD', 'STFIPS', 'DSCODE', 'COCODE', 'HUCODE',
     $     'STFIPS', 'DSCODE',    4*'      ',
     $     'PARMCD', 'STATCD', 'SEASBG', 'SEASND',    2*'      ',
     $        56,       57,       41,       42,        6,        4,
     $        41,       42,       4*0,
     $        56,       57,      256,      257,       2*0 /
C          additional real attributes (name, index)
      DATA RNAM, RNDX
     $   / 'LATDEG', 'LNGDEG', 'DAREA ', 'CONTDA', 'DATUM ', 'WELLDP',
     $     'BASEQ ',   7*'      ',
     $     'XSECLC', 'DEPTH ', 'LATDEG', 'LNGDEG', 3*'      ',
     $         8,        9,       11,       43,      264,       47,
     $        49,      7*0,
     $       258,      259,         8,        9,      3*0 /
C          additional character attributes (name, index, length)
      DATA CNAM, CNDX, CLEN
     $   / 'SITECO', 'AQTYPE', 'GUCODE', 'AGENCY',
     $     'AGENCY', 3*'      ',  4*'      ',
     $        44,       48,       46,       40,
     $        40,      3*0,      4*0,
     $         4,        4,        8,        8,
     $         8,      3*0,      4*0 /
C          missing value indicators for attributes
      DATA IMISS, RMISS,  CMISS
     $     /-999, -999.,   ' ' /
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT ( 6A1 )
C3001 format (//, ' <-- initat:  aver =  ',  i5,
C    $         /, '              aimx =  ',  i5,
C    $         /, '              armx =  ',  i5,
C    $         /, '              acmx =  ',  i5 )
C3002 format (//, '              aindx:  ', 10i5, /, 22x, 9i5,
C    $         /, '              arndx:  ',  8i5,
C    $         /, '              acndx:  '   7i5,
C    $         /, '              aclen:  ',  7i5 )
C3003 format (//, '              aival:  ',  5i10, /, 22x, 5i10, 
C    $         /, '                      ',  5i10, /, 22x, 4i10,
C    $         /, '              arval:  ',  5f10.0, /, 22x, 3f10.0,
C    $         /, '              acval:  ',  50a1, /22x, 42a1 )
C
C     + + + END SPECIFICATIONS + + +
C
C     save version
      AVER = VERS
C     set maximums for each shared atribute type
      AIMX = IMXS
      ARMX = RMXS
      ACMX = CMXS
C
C     initialize the arrays of indexes and length for shared attributes
      CALL COPYI ( AIMX, INDXS, AINDX )
      CALL COPYI ( ARMX, RNDXS, ARNDX )
      CALL COPYI ( ACMX, CNDXS, ACNDX )
      CALL COPYI ( ACMX, CLENS, ACLEN )
C
C     initialize the names arrays for shared attributes
      DO 110 I = 1, AIMX
        READ (INAMS(I),1000) (AINAME(J,I), J = 1, 6)
 110  CONTINUE
      DO 120 I = 1, ARMX
        READ (RNAMS(I),1000) (ARNAME(J,I), J = 1, 6)
 120  CONTINUE
      DO 130 I = 1, ACMX
        READ (CNAMS(I),1000) (ACNAME(J,I), J = 1, 6)
 130  CONTINUE
C
      IF (VERS .GT. 0  .AND.  VERS .LE. MXVS) THEN
C       recognized version, add any version specific attributes
        IF (AMXV(1,VERS) .GT. 0) THEN
C         additional integer attributes
          LPT = AIMX + 1
          AIMX = AIMX + AMXV(1,VERS)
          CALL COPYI ( AMXV(1,VERS), INDX(1,VERS), AINDX(LPT) )
          I = 0
          DO 210 L = LPT, AIMX
            I = I + 1
            READ (INAM(I,VERS),1000) (AINAME(J,L), J = 1, 6)
 210      CONTINUE
        END IF
        IF (AMXV(2,VERS) .GT. 0) THEN
C         additional real attributes
          LPT = ARMX + 1
          ARMX = ARMX + AMXV(2,VERS)
          CALL COPYI ( AMXV(2,VERS), RNDX(1,VERS), ARNDX(LPT) )
          I = 0
          DO 220 L = LPT, ARMX
            I = I + 1
            READ (RNAM(I,VERS),1000) (ARNAME(J,L), J = 1, 6)
 220      CONTINUE
        END IF
        IF (AMXV(3,VERS) .GT. 0) THEN
C         additional character attributes
          LPT = ACMX + 1
          ACMX = ACMX + AMXV(3,VERS)
          CALL COPYI ( AMXV(3,VERS), CNDX(1,VERS), ACNDX(LPT) )
          CALL COPYI ( AMXV(3,VERS), CLEN(1,VERS), ACLEN(LPT) )
          I = 0
          DO 230 L = LPT, ACMX
            I = I + 1
            READ (CNAM(I,VERS),1000) (ACNAME(J,L), J = 1, 6)
 230      CONTINUE
        END IF
      END IF
C     zero or blank out attribute values
      CALL ZIPI ( AIMX, IMISS, AIVAL )
      CALL ZIPR ( ARMX, RMISS, ARVAL )
      CALL ZIPC ( ACLONG, CMISS, ACVAL )
C     special cases
C     tsfill
      ARVAL(1) = -99999.
C     space allocations
      I = 5
      CALL COPYI ( I, SPACE, AIVAL(8) )
C     time series attributes
      I = 7
      CALL COPYI ( I, TIME, AIVAL(1) )
C     write (99,3001) aver, aimx, armx, acmx
C     write (99,3002) aindx, arndx, acndx, aclen
C     write (99,3003) aival, arval, acval
C
      RETURN
      END
C
C
C
      SUBROUTINE   INATRB
     I                   ( MESSFL, FE, WDMFL, DSN,
     O                     RET )
C
C     + + + PURPOSE + + +
C     Adds attributes to a data set.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, FE, WDMFL, DSN, RET
C
C     + + + ARGUMENT DEFINITION + + +
C     MESSFL - Fortran unit number of the message file
C     FE     - Fortran unit number of error file
C     WDMFL  - Fortran unit number of the wdm file
C     DSN    - data-set number for attributes
C     RET   - return code
C
C     + + + COMMONS + + +
      INCLUDE 'catrib.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   K, J, LEN, RETC, I, LNA
C
C     + + + LOCAL DEFINITIONS + + +
C     RETC   - return code from putting attributes
C               -81 - data set does not exist
C              -101 - incorrect character value for attribute
C              -102 - attribute already in data set
C              -103 - no room in data set for attribute
C              -104 - data present, can't update attribute
C              -105 - attribute not allowed for this type data set
C                     ignored, assumes that time attribute to table ds
C              -108 - incorrect integer value for attribute
C              -109 - incorrect real value for attribute
C
C     + + + FUNCTIONS + + +
      INTEGER   LENSTR
C
C     + + + INTRINSICS + + +
      INTRINSIC  ABS
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDBSAI, WDBSAR, WDBSAC, LENSTR
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (//,2X, 'DSN', I6, ':    adding attributes',
     $        //,2X, '              Name       Value' )
 2001 FORMAT (   2X, '              ', 6A1,     I10 )
 2002 FORMAT (   2X, '              ', 6A1, 2X, G12.6 )
 2003 FORMAT (   2X, '              ', 6A1, 6X, 48A1 )
 2011 FORMAT (   2X, '    --->      ', 6A1, 2X, 'return code = ', I4,
     $               ',  value = ', I10 )
 2012 FORMAT (   2X, '    --->      ', 6A1, 2X, 'return code = ', I4,
     $               ',  value = ', G12.6 )
 2013 FORMAT (   2X, '    --->      ', 6A1, 2X, 'return code = ', I4,
     $               ',  value = ', 48A1 )
C
C3000 format (//, ' --> inatrb:   dsn =  ',  i5,
C    $         /, '              aver =  ',  i5,
C    $         /, '              aimx =  ',  i5,
C    $         /, '              armx =  ',  i5,
C    $         /, '              acmx =  ',  i5 )
C3001 format (//, ' <-- inatrb:  aver =  ',  i5,
C    $         /, '              aimx =  ',  i5,
C    $         /, '              armx =  ',  i5,
C    $         /, '              acmx =  ',  i5 )
C3002 format (//, '              aindx:  ', 10i5, /, 22x, 9i5,
C    $         /, '              arndx:  ',  8i5,
C    $         /, '              acndx:  '   7i5,
C    $         /, '              aclen:  ',  7i5 )
C3003 format (//, '              aival:  ',  5i10, /, 22x, 5i10, 
C    $         /, '                      ',  5i10, /, 22x, 4i10,
C    $         /, '              arval:  ',  5f10.0, /, 22x, 3f10.0,
C    $         /, '              acval:  ',  50a1, /22x, 42a1 )
C
C     + + + END SPECIFICATIONS + + +
C
C     write (99,3000) dsn, aver, aimx, armx, acmx
C     write (99,3002) aindx, arndx, acndx, aclen
C     write (99,3003) aival, arval, acval
C     write out heading and dsn
      WRITE (FE,2000) DSN
C
      RET = 0
      IF (AIMX .GT. 0) THEN
C       add integer attributes
        RET = 0
        LEN = 1
        DO 100 K = 1, AIMX
          IF (AIVAL(K) .NE. -999  .AND.  AINDX(K) .GT. 0) THEN
C           valid integer attribute
            CALL WDBSAI ( WDMFL, DSN, MESSFL, AINDX(K), LEN, AIVAL(K),
     O                    RETC )
            IF (RETC .EQ. 0) THEN
C             document add
              WRITE (FE,2001) (AINAME(J,K),J=1,6), AIVAL(K)
            ELSE IF (RETC .EQ. -105  .OR.
     $               RETC .EQ. -102  .OR.  RETC .EQ. -104) THEN
C             wrong ds type, assume time attribute with table data set
C             already present or data present and can't update
C             ignore the warning
            ELSE
C             error in putting attribute
              WRITE (FE,2011) (AINAME(J,K),J=1,6), RETC, AIVAL(K)
              RET = RET + 1
            END IF
CPRH        looks like this is not needed
CPRH            IF (RETC .NE. 0) THEN
CPRHC             error adding attribute
CPRH              RET = RET + 1
CPRH            END IF
          END IF
 100    CONTINUE
      END IF
C
      IF (ARMX .GT. 0) THEN
C       add real attributes
        LEN = 1
        DO 200 K = 1, ARMX
          IF ((ABS(ARVAL(K)+999.) .GT. .0001 .AND. ARNDX(K) .GT. 0) .OR.
     $        ARNDX(K) .EQ. 32) THEN
C           valid real attribute (32 is tsfill)
            CALL WDBSAR ( WDMFL, DSN, MESSFL, ARNDX(K), LEN, ARVAL(K),
     O                    RETC )
            IF (RETC .EQ. 0) THEN
C             document add
              WRITE (FE,2002) (ARNAME(J,K),J=1,6), ARVAL(K)
            ELSE IF (RETC .EQ. -102  .OR.  RETC .EQ. -104) THEN
C             already present or data present and can't update
C             ignore warning
            ELSE
C             error in putting attribute
              WRITE (FE,2012) (ARNAME(J,K),J=1,6), RETC, ARVAL(K)
              RET = RET + 1
            END IF
          END IF
 200    CONTINUE
      END IF
C
      IF (ACMX .GT. 0) THEN
C       add character attributes
        I = 1
        DO 300 K = 1, ACMX
          LEN = ACLEN(K)
          IF (ACNDX(K) .GT. 0  .AND.  LEN .GT. 0) THEN
C           attribute value could be here
            LNA = LENSTR ( LEN, ACVAL(I) )
          ELSE
C           no attribute is defined
            LNA = 0
          END IF
          IF (LNA .GT. 0) THEN
C           attribute is present
            CALL WDBSAC ( WDMFL, DSN, MESSFL, ACNDX(K), LEN, ACVAL(I),
     O                    RETC)
            IF (RETC .EQ. 0) THEN
C             document add
              WRITE (FE,2003) (ACNAME(J,K),J=1,6), 
     $                        (ACVAL(J),J=I,I+LEN-1)
            ELSE IF (RETC .EQ. -101 .AND.
     $        ACVAL(I) .EQ. 'n'    .AND.  ACVAL(I+1) .EQ. 'o' .AND.
     $        ACVAL(I+2) .EQ. 'n'  .AND.  ACVAL(I+3) .EQ. 'e') THEN
C             invalid charater type is 'none', assume not defined & ignore
              RET = RET + 0
            ELSE IF (RETC .EQ. -102  .OR.  RETC .EQ. -104) THEN
C             already present or data present and can't update
C             ignore the warning
            ELSE
C             error in putting attribute
              WRITE (FE,2013) (ACNAME(J,K),J=1,6), RETC,
     $                        (ACVAL(J),J=I,I+LEN-1)
              RET = RET + 1
            END IF
          END IF
C         update position in attribute array
          I = I + LEN
 300    CONTINUE
      END IF
C     write (99,3001) aver, aimx, armx, acmx
C     write (99,3002) aindx, arndx, acndx, aclen
C     write (99,3003) aival, arval, acval
C
      RETURN
      END
C
C
C
      SUBROUTINE   INITAX
C
C     + + + PURPOSE + + + 
C     Initialize attribute values to no value.
C
C     + + + INCLUDES + + +
      INCLUDE 'catrib.inc'
C
C     + + + PARAMETERS + + +
      INTEGER    MXVS
      PARAMETER (MXVS = 4)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      VERS, IPT, MISSI, AMXV(3,MXVS), IPTA(3)
      REAL         MISSR
      CHARACTER*1  BLANK
C
C     + + + LOCAL DEFINITIONS + + + 
C     IPTA   - starting location in attribute value arrays for
C              attributes that need to be initialized to none.
C              (1) - integer, the first 12 integer attributes and
C                    space allocations are not re-initialized.
C              (2) - real, the first, tsfill, is not re-initialized
C              (3) - character, the last, agency is not re-initialized
C     AMXV   - number of integer, real and character attributes to
C              be reinitialized for each attribute.  The version number
C              is v:
C              (1,v) - number of integer attributes
C              (2,v) - number of real attributes
C              (3,v) - total length of character attributess
C
C     + + + EXTERNALS + + +
      EXTERNAL   ZIPI, ZIPR, ZIPC
C
C     + + + DATA INITIALIZATIONS + + +
      DATA   MISSI, MISSR, BLANK
     $     /  -999, -999.,   ' ' /
      DATA   IPTA,     AMXV
     $     / 13,2,1,   7,7,84, 3,0,68, 5,4,68, 1,0,68 /
C
C     + + + OUTPUT FORMATS + + +
C3000 format (//, ' --> initax:  aver =  ',  i5,
C    $         /, '              aimx =  ',  i5,
C    $         /, '              armx =  ',  i5,
C    $         /, '              acmx =  ',  i5 )
C3001 format (//, ' <-- initax:  aver =  ',  i5,
C    $         /, '              aimx =  ',  i5,
C    $         /, '              armx =  ',  i5,
C    $         /, '              acmx =  ',  i5 )
C3002 format (//, '              aindx:  ', 10i5, /, 22x, 9i5,
C    $         /, '              arndx:  ',  8i5,
C    $         /, '              acndx:  '   7i5,
C    $         /, '              aclen:  ',  7i5 )
C3003 format (//, '              aival:  ',  5i10, /, 22x, 5i10, 
C    $         /, '                      ',  5i10, /, 22x, 4i10,
C    $         /, '              arval:  ',  5f10.0, /, 22x, 3f10.0,
C    $         /, '              acval:  ',  50a1, /22x, 42a1 )
C
C     + + + END SPECIFICATIONS + + +
C
C     write (99,3000) aver, aimx, armx, acmx
C     write (99,3002) aindx, arndx, acndx, aclen
C     write (99,3003) aival, arval, acval
      VERS = AVER
      IF (VERS .LT. 1  .OR.  VERS .GT. MXVS) THEN
C       version is not recognized, default to mininum
        VERS = MXVS
      END IF
      IF (AMXV(1,VERS) .GT. 0) THEN
C       integer attributes
        IPT = IPTA(1)
        CALL ZIPI ( AMXV(1,VERS), MISSI, AIVAL(IPT) )
      END IF
      IF (AMXV(2,VERS) .GT. 0) THEN
C       real attributes
        IPT = IPTA(2)
        CALL ZIPR ( AMXV(2,VERS), MISSR, ARVAL(IPT) )
      END IF
      IF (AMXV(3,VERS) .GT. 0) THEN
C       character attributes
        IPT = IPTA(3)
        CALL ZIPC ( AMXV(3,VERS), BLANK, ACVAL(IPT) )
      END IF
C     write (99,3001) aver, aimx, armx, acmx
C     write (99,3002) aindx, arndx, acndx, aclen
C     write (99,3003) aival, arval, acval
C
      RETURN
      END
C
C
C
      SUBROUTINE   CKSPCE
     I                   ( NSA, NSP, NDP, NDN, NUP,
     O                     RETC )
C
C     + + + PURPOSE + + +
C     This subroutine checks that the values specified for data-set
C     space allocation are valid.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NSA, NSP, NDP, NDN, NUP, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NSA    - number of search attributes
C     NSP    - space for search attributes, note that the space for
C              attributes must be at least as large as the number of
C              attributes.  Some attributes (ie staid, stanam) require
C              extra space.
C     NDP    - space for data group pointers
C     NDN    - number of down pointers to data sets
C     NUP    - number of up pointers to data sets
C     RETC   - return code
C                 0 - numbers look ok
C              -141 - numbers of attributes and data groups ok, but more
C                     attribute space requested then can be used
C              -142 - problem with number of attributes or data groups
C              -143 - number of attributes exceeds space for attributes
C              -144 - total space too big
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   SPACE, SPACMX, SPACAT, NSAMN, NSAMX, NDPMN, NDPMX
C
C     + + + LOCAL DEFINITIONS + + +
C     SPACMX - maximum space available for attributes, data group
C              pointers, and up & down data-set pointers
C              fixed space required for pointers and counters
C                 12 = 1-12 pointers
C                  2 = counts for up and down data set pointers
C                  4 = attr count & pointer, data count & pointer
C                 ?? = minimum space required for first data group
C                 --
C                 18
C              spacmx = record size - fixed space - first data blocks
C                     = 512 - 18 - 20 = 474
C     SPACAT - extra space that might be needed for attribute arrays
C              staid (3), stanam (11), descrp (19), agency (1)
C              grpnam (1), gucode (1)
C              36 is about 40
C     NSAMN  - minimum value for number of search attributes (NSA)
C     NSAMX  - maximum value for number of search attributes (NSA)
C     NDPMN  - minimum value for number of data groups (NDP)
C     NDPMX  - maximum value for number of data groups (NDP)
C     SPACE  - space allocated for attributes, data group pointers,
C              and up & down data-set pointers
C                ndn = down data set pointers
C                nup = up data set pointers
C              2*nsa = attribute indexes and pointers
C                nsp = space for attributes
C                ndp = data group pointers
C              
C     + + + DATA INITIALIZASTIONS + + +
C                     attributes   data groups
      DATA   SPACMX, SPACAT,  NSAMN, NSAMX, NDPMN, NDPMX
     $     /    474,     40,      8,   150,     1,   400 /
C
C     + + + END SPECIFICATIONS + + +
C
C     calculate space requested
      SPACE = 2 * NSA  +  NSP  +  NDP  +  NDN  +  NUP
C
C     is there enough room?
      IF (SPACE .LE. SPACMX) THEN
C       total space is small enough
        IF (NSP .GE. NSA) THEN
C         enough space for the number of attributes
          IF (NSAMN .LE. NSA  .AND.  NSA .LE. NSAMX  .AND.
     $        NDPMN .LE. NDP  .AND.  NDP .LE. NDPMX) THEN
C           number of attributes and data groups ok
            IF (NSP .LE. NSA+SPACAT) THEN
C             reasonable amount of space for number of attributes
              RETC = 0
            ELSE
C             requested more attribute space then can be used
              RETC = -141
            END IF
          ELSE
C           problem with number of attributes or data groups
            RETC = -142
          END IF
        ELSE
C         number of attributes exceeds space for attributes
          RETC = -143
        END IF
      ELSE
C       total space is too big
        RETC = -144
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   INWIOX
     I                    ( INTFLG )
C
C     + + + PURPOSE + + +
C     Specifies interaction for wdm input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   INTFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INTFLG - Interactive (1) or non-interactive (0)
C
C     + + + INCLUDES + + +
      INCLUDE 'ciowdm.inc'
C
C     + + + END SPECIFICATIONS + + +
C
C     set interactive flag in common
      INTFG = INTFLG
C
      RETURN
      END
