#!/bin/sh
# test.sh -- run iowdm test data sets
#
# Usage: test.sh [start [stop]] [| tee test.out]
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#
# History: 96/10/08  kmflynn
#         
# pathnames
  WrdA=/usr/opt/wrdapp
  Iowd=$WrdA/iowdm4.1
  Anne=$WrdA/annie4.1

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

  Data=$Iowd/data
  Chck=$Iowd/test

  exec 2>&1                          # stderr shows up in .out file
  Start=${1-1}                       # by default, start at 1
  Stop=${2-6}                        # by default, stop at 6
                                     # 0 - file build
                                     # 1 - input attributes
                                     # 2 - input time series
                                     # 3 - input basin and n-day
                                     # 4 - input basin and peaks
                                     # 5 - input daily
                                     # 6 - input unit
                                     # 7 - annie export all data sets

# begin test runs
  if [ -f test.out ] ; then rm test.out ; fi
  echo                                                        | tee    test.out
  echo ______________________________________________________ | tee -a test.out
  echo                                                        | tee -a test.out
  echo " `date` "                                             | tee -a test.out
  echo                                                        | tee -a test.out
  echo " Processing iowdm test runs $Start to $Stop"          | tee -a test.out
  echo                                                        | tee -a test.out
  echo "  iowdm program from: $Iowd"                          | tee -a test.out
  echo "  annie program from: $Anne"                          | tee -a test.out
  echo "      test data from: $Data"                          | tee -a test.out
  echo

  Test=$Start

  if [ $Test -eq 0  -o  ! -f test.wdm ] ; then
#    build a new test.wdm file
     echo                                                     | tee -a test.out
     echo                                                     | tee -a test.out
     echo ___________________________________________________ | tee -a test.out
     echo "Test run number $Test - build wdm file"            | tee -a test.out
     echo                                                     | tee -a test.out
#    delete old if present
     if [ -f test.wdm ] ; then rm test.wdm ; fi
     $Iowd/bin/iowdm <<-EOT
	@$Data/test0.log
	EOT
     mv ERROR.FIL test0.err
     if [ $Test -eq 0 ] ; then Test=`expr $Test + 1` ; fi
  fi
     
  while [ $Test -ge $Start -a $Test -le $Stop -a $Test -le 6 ] ; do
#    iowdm test runs
     echo                                                     | tee -a test.out
     echo                                                     | tee -a test.out
     echo ___________________________________________________ | tee -a test.out
     echo "Test run number $Test"                             | tee -a test.out
     echo                                                     | tee -a test.out
     FileO='IOWDM.LOG ERROR.FIL'
     Sufix='gsd gsu bcd pks ndy ft1 ft2 lfl tfl'
     for File in test$Test.out $FileO ; do
#       remove old files
        if [ -f $File ] ; then rm $File ; fi
     done
     for File in $Sufix ; do
#       remove old files and link in current versions
        if [ -f test$Test.$File ]       ; then rm test$Test.$File ; fi
        if [ -f $Data/test$Test.$File ] ; then
           ln -s $Data/test$Test.$File test$Test.$File
        fi
     done

     $Iowd/bin/iowdm <<-EOT
	@$Data/test$Test.log
	EOT

#    save error/summary file, remove input files
     mv ERROR.FIL test$Test.err
     for Fils in gsd gsu bcd pks ndy ft1 ft2 lfl tfl ; do
        if [ -f test$Test.$Fils ] ; then rm test$Test.$Fils ; fi
     done

     Test=`expr $Test + 1`
  done

  if [ $Test -eq 7 ] ; then
#    export data sets using annie for verification
     echo                                                     | tee -a test.out
     echo                                                     | tee -a test.out
     echo ___________________________________________________ | tee -a test.out
     echo "Test run number $Test export data sets"            | tee -a test.out
     echo                                                     | tee -a test.out
     if [ -f test7.exp ] ; then rm test7.exp ;fi
     $Anne/bin/annie <<-EOT
	@$Data/test7.log
	EOT

     mv ERROR.FIL test$Test.err
  fi

  echo                                                        | tee -a test.out
  echo                                                        | tee -a test.out
  echo ______________________________________________________ | tee -a test.out
  echo "Completed IOWDM test runs $Start to $Stop"            | tee -a test.out
  echo                                                        | tee -a test.out
# check output against original output in $Data directory
  $Chck/check.sh $Data

# end of shell
